@@ -41,7 +41,6 @@
 #include "terminal-defines.h"
 #include "terminal-prefs.h"
 
-#include <sys/wait.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
@@ -267,30 +266,19 @@ app_menu_help_cb (GSimpleAction *action,
                   GVariant      *parameter,
                   gpointer       user_data)
 {
-  terminal_util_show_help (NULL, NULL);
+        GtkApplication *application = user_data;
+
+        terminal_util_show_help (NULL, gtk_application_get_active_window (application));
 }
 
 static void
 app_menu_about_cb (GSimpleAction *action,
                    GVariant      *parameter,
                    gpointer       user_data)
-{
-  terminal_util_show_about (NULL);
-}
-
-static void
-app_menu_quit_cb (GSimpleAction *action,
-                  GVariant      *parameter,
-                  gpointer       user_data)
 {
   GtkApplication *application = user_data;
-  GtkWindow *window;
 
-  window = gtk_application_get_active_window (application);
-  if (TERMINAL_IS_WINDOW (window))
-    terminal_window_request_close (TERMINAL_WINDOW (window));
-  else /* a dialogue */
-    gtk_widget_destroy (GTK_WIDGET (window));
+  terminal_util_show_about (gtk_application_get_active_window (application));
 }
 
 /* Class implementation */
@@ -311,8 +299,7 @@ terminal_app_startup (GApplication *application)
   const GActionEntry app_menu_actions[] = {
     { "preferences", app_menu_preferences_cb,   NULL, NULL, NULL },
     { "help",        app_menu_help_cb,          NULL, NULL, NULL },
-    { "about",       app_menu_about_cb,         NULL, NULL, NULL },
-    { "quit",        app_menu_quit_cb,          NULL, NULL, NULL }
+    { "about",       app_menu_about_cb,         NULL, NULL, NULL }
   };
 
   GtkBuilder *builder;
@@ -357,10 +344,6 @@ terminal_app_init (TerminalApp *app)
 
   /* Terminal global settings */
   app->global_settings = g_settings_new (TERMINAL_SETTING_SCHEMA);
-  g_settings_bind (app->global_settings, TERMINAL_SETTING_DARK_THEME_KEY,
-                   gtk_settings_get_default (),
-                   "gtk-application-prefer-dark-theme",
-                   G_SETTINGS_BIND_GET);
 
   /* Check if we need to migrate from gconf to dconf */
   maybe_migrate_settings (app);
@@ -377,7 +360,7 @@ terminal_app_init (TerminalApp *app)
                     app);
 
   settings = g_settings_get_child (app->global_settings, "keybindings");
-  terminal_accels_init (G_APPLICATION (app), settings);
+  terminal_accels_init (settings);
   g_object_unref (settings);
 }
 
