@@ -3,7 +3,7 @@
  * Copyright © 2002 Red Hat, Inc.
  * Copyright © 2002 Sun Microsystems
  * Copyright © 2003 Mariano Suarez-Alvarez
- * Copyright © 2011 Christian Persch
+ * Copyright © 2011, 2013 Christian Persch
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,8 @@
 
 #include <gio/gio.h>
 
-#if GDK_WINDOWING_X11
+#include <gdk/gdk.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #endif
 
@@ -137,7 +138,7 @@ terminal_client_append_exec_options (GVariantBuilder *builder,
 void 
 terminal_client_get_fallback_startup_id  (char **startup_id)
 {
-#if GDK_WINDOWING_X11
+#ifdef GDK_WINDOWING_X11
   GdkDisplay *display;
   Display *xdisplay;
   Window xwindow;
